@@ -1,12 +1,12 @@
 import os, ConfigParser, cgi, re
 import glob
-from os.path import join, isfile, abspath, splitext, expanduser, exists
+from os.path import join, isfile, abspath, splitext, expanduser, exists, isdir
 from gettext import gettext as _
 from deskbar.defs import VERSION
 import gobject
 import gtk
 import deskbar, deskbar.Indexer, deskbar.Utils
-import deskbar.Handler, deskbar.gnomedesktop
+import deskbar.Handler, deskbar.Match, deskbar.gnomedesktop
 from deskbar.Utils import get_xdg_data_dirs
 
 HANDLERS = {
@@ -32,6 +32,9 @@ HANDLERS = {
 	},
 }
 
+EXACT_MATCH_PRIO = 100
+EXACT_WORD_PRIO = 50
+
 class GenericProgramMatch(deskbar.Match.Match):
 	def __init__(self, backend, use_arg=False, desktop=None, desktop_file=None, **args):
 		deskbar.Match.Match.__init__(self, backend, **args)
@@ -39,6 +42,7 @@ class GenericProgramMatch(deskbar.Match.Match):
 		self.desktop_file = desktop_file
 		self.use_arg = use_arg
 		
+		self._priority = EXACT_WORD_PRIO
 		self._icon = deskbar.Utils.load_icon_for_desktop_icon(self.icon)
 		self._desktop = desktop
 		if desktop == None:
@@ -125,6 +129,7 @@ class SpecialProgramHandler(deskbar.Handler.Handler):
 		
 	def query(self, qstring):
 		if self._match != None:
+			self._match._priority = get_priority_for_name(qstring, self._match._desktop.get_string("Exec"))
 			return [self._match]
 		else:
 			return []
@@ -164,6 +169,53 @@ class DevhelpHandler(SpecialProgramHandler):
 					icon=desktop.get_string(deskbar.gnomedesktop.KEY_ICON),
 					desktop=desktop,
 					desktop_file=f)
+
+class PathProgramMatch(deskbar.Match.Match):
+	def __init__(self, backend, name=None, use_terminal=False, priority=0, **args):
+		deskbar.Match.Match.__init__(self, backend, name=name, **args)
+		self.use_terminal = use_terminal
+		self._priority = EXACT_MATCH_PRIO
+		
+	def set_with_terminal(self, terminal):
+		self.use_terminal = terminal
+		
+	def get_hash(self, text=None):
+		if not self.use_terminal:
+			return text
+		else:
+			return (text, True)
+		
+	def action(self, text=None):
+		if self.use_terminal:
+			try:
+				prog = subprocess.Popen(
+					text.split(" "),
+					stdout=subprocess.PIPE,
+					stderr=subprocess.STDOUT)
+				
+				zenity = subprocess.Popen(
+					["zenity", "--title="+text,
+						"--window-icon="+join(deskbar.ART_DATA_DIR, "generic.png"),
+						"--width=700",
+						"--height=500",
+						"--text-info"],
+					stdin=prog.stdout)
+	
+				# Reap the processes when they have done
+				gobject.child_watch_add(zenity.pid, lambda pid, code: None)
+				gobject.child_watch_add(prog.pid, lambda pid, code: None)
+				return
+			except:
+				#No zenity, get out of the if, and launch without GUI
+				pass
+		
+		gobject.spawn_async(text.split(" "), flags=gobject.SPAWN_SEARCH_PATH)			
+
+	def get_category(self):
+		return "actions"
+	
+	def get_verb(self):
+		return _("Execute %s") % "<b>%(text)s</b>"
 		
 class ProgramsHandler(deskbar.Handler.Handler):
 	def __init__(self):
@@ -172,10 +224,45 @@ class ProgramsHandler(deskbar.Handler.Handler):
 		
 	def initialize(self):
 		self._scan_desktop_files()
+		self._path = [path for path in os.getenv("PATH").split(os.path.pathsep) if path.strip() != "" and exists(path) and isdir(path)]
 		
 	def query(self, query):
-		return self._indexer.look_up(query)[:deskbar.DEFAULT_RESULTS_PER_HANDLER]
+		result = self.query_path_programs(query)
+		result += self.query_desktop_programs(query)
+		return result
 		
+	def query_path_programs(self, query):
+		args = query.split(" ")
+		match = self._check_program(args[0])
+
+		if match != None:
+			return [match]
+		else:
+			return []
+	
+	def query_desktop_programs(self, query):
+		result = []
+		for match in self._indexer.look_up(query):
+			match._priority = get_priority_for_name(query, match._desktop.get_string("Exec"))
+			result.append(match)
+		return result
+			
+	
+	def on_key_press(self, query, shortcut):
+		if shortcut == gtk.keysyms.t:
+			match = self._check_program(query.split(" ")[0])
+			if match != None:
+				match.set_with_terminal(True)
+				return match
+			
+		return None
+		
+	def _check_program(self, program):
+		for path in self._path:
+			prog_path = join(path, program)
+			if exists(prog_path) and isfile(prog_path):
+				return PathProgramMatch(self, program)	
+				
 	def _scan_desktop_files(self):
 		for dir in get_xdg_data_dirs():
 			for f in glob.glob(join(dir, "applications", "*.desktop")):
@@ -195,6 +282,12 @@ class ProgramsHandler(deskbar.Handler.Handler):
 								result.get_string(deskbar.gnomedesktop.KEY_COMMENT),
 							), match)
 
+def get_priority_for_name(query, name):
+	if name.split(" ")[0].endswith(query):
+		return EXACT_MATCH_PRIO
+	else:
+		return EXACT_WORD_PRIO
+		
 def parse_desktop_filename(desktop, only_if_visible=True):
 	if desktop[0] == "/" and exists(desktop):
 		return parse_desktop_file(desktop, only_if_visible)
